#include <stdio.h>

#include <powwow/defines.h>
#include <powwow/cmd.h>

#include <powwow/eval.h>
#include <powwow/list.h>
#include <powwow/main.h>

#include "jsapi.h"

void javascript_exec( char *arg );
JSBool javascript_powwow_execute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
JSBool javascript_powwow_var(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);

cmdstruct js_command = { NULL, "javascript", "Execute javascript", javascript_exec, NULL };

JSRuntime *js_rt;
JSContext *js_cx;
JSObject  *js_glob, *js_powwow;
JSClass js_global_class = {
	"global",0,
	JS_PropertyStub,JS_PropertyStub,JS_PropertyStub,JS_PropertyStub,
	JS_EnumerateStub,JS_ResolveStub,JS_ConvertStub,JS_FinalizeStub
};

void powwow_init() {
	printf( "Spidermonkey javascript engine plugin for powwow v" VERSION "\n"
		"Author: Steve Slaven - http://hoopajoo.net\n" );

	// init runtime
	js_rt = JS_NewRuntime(8*1024*1024); // 8M virtual machine (runtime opt?)
	if( ! js_rt ) {
		printf( "Error initializing javascript\n" );
		return;
	}

	// Setup context
	js_cx = JS_NewContext(js_rt, 8192);
	if( js_cx == NULL ) {
		printf( "Error initializing js context\n" );
		return;
	}

	// init globals
	js_glob = JS_NewObject( js_cx, &js_global_class, NULL, NULL );
	// and builtins
	JS_InitStandardClasses( js_cx, js_glob );
	// Add the powwow object
	js_powwow = JS_DefineObject( js_cx, js_glob, "powwow", NULL, NULL, 
			JSPROP_ENUMERATE | JSPROP_PERMANENT );

	JS_DefineFunction( js_cx, js_powwow, "execute", javascript_powwow_execute, 1, NULL );
	JS_DefineFunction( js_cx, js_powwow, "get", javascript_powwow_var, 1, NULL );

	// Add command
	cmd_add_command( &js_command );
}

void javascript_exec( char *arg ) {
	jsval rval;
	JSString *str;
	JSBool ok;

	ok = JS_EvaluateScript(js_cx, js_glob, arg, strlen(arg),
		"internal", 0, &rval);
	str = JS_ValueToString(js_cx, rval);
	//printf("script result: %s\n", JS_GetStringBytes(str));
}

JSBool javascript_powwow_execute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
	JSString *str = JS_ValueToString(js_cx, argv[ 0 ] );

	printf( "Execute: '%s'\n", JS_GetStringBytes(str) );
	parse_user_input( JS_GetStringBytes(str), 0 );
	return( JS_TRUE );
}

JSBool javascript_powwow_var(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
	JSString *str = JS_ValueToString(js_cx, argv[ 0 ] );
	char *varname = JS_GetStringBytes( str );
	JSString *rstr;
	int idx;
	varnode *v = NULL;
	char retval[ 1024 ];

	bzero( retval, 1024 );
	/* printf( "Lookup var: '%s'\n", JS_GetStringBytes(str) ); */

	/* DWIM, get the global var or named var based on if char1 is a digit */
	if( isdigit( varname[ 1 ] ) ) {
		sscanf( &varname[ 1 ], "%d", &idx );
		if( idx < NUMVAR ) {
                	if( varname[ 0 ] == '@' ) {
				sprintf( retval, "%d", *VAR[ idx ].num );
			}else if( varname[ 0 ] == '$' ) {
				strncpy( retval, ptrdata( *VAR[ idx ].str ), 1024 );
			}
		}else{
			sprintf( retval, "Global variable index out of range" );
		}
	}else{
		if( varname[ 0 ] == '@' ) {
			v = *lookup_varnode( (char *)(varname + 1), 0 );
			if( v != NULL )
				sprintf( retval, "%d", v -> num );
		}else if( varname[ 0 ] == '$' ) {
			v = *lookup_varnode( (char *)(varname + 1), 1 );
			if( v != NULL )
				strncpy( retval, ptrdata( v -> str ), 1024 );
		}
	}

	/* printf( "Found value: '%s'\n", retval ); */
	rstr = JS_NewStringCopyZ(cx, retval);
	if( rstr )
		*rval = STRING_TO_JSVAL( rstr );
	
	return( JS_TRUE );
}
